@@ -1,4 +1,4 @@
-/* Copyright (c) 2013-2017 the Civetweb developers
+/* Copyright (c) 2013-2020 the Civetweb developers
  * Copyright (c) 2013 No Face Press, LLC
  *
  * License http://opensource.org/licenses/mit-license.php MIT License
@@ -27,6 +27,19 @@ CivetHandler::handleGet(CivetServer *server, struct mg_connection *conn)
 	return false;
 }
 
+bool
+CivetHandler::handleGet(CivetServer *server,
+                        struct mg_connection *conn,
+                        int *status_code)
+{
+	UNUSED_PARAMETER(server);
+	UNUSED_PARAMETER(conn);
+	if (status_code) {
+		*status_code = -1;
+	}
+	return false;
+}
+
 bool
 CivetHandler::handlePost(CivetServer *server, struct mg_connection *conn)
 {
@@ -35,6 +48,19 @@ CivetHandler::handlePost(CivetServer *server, struct mg_connection *conn)
 	return false;
 }
 
+bool
+CivetHandler::handlePost(CivetServer *server,
+                         struct mg_connection *conn,
+                         int *status_code)
+{
+	UNUSED_PARAMETER(server);
+	UNUSED_PARAMETER(conn);
+	if (status_code) {
+		*status_code = -1;
+	}
+	return false;
+}
+
 bool
 CivetHandler::handleHead(CivetServer *server, struct mg_connection *conn)
 {
@@ -43,6 +69,19 @@ CivetHandler::handleHead(CivetServer *server, struct mg_connection *conn)
 	return false;
 }
 
+bool
+CivetHandler::handleHead(CivetServer *server,
+                         struct mg_connection *conn,
+                         int *status_code)
+{
+	UNUSED_PARAMETER(server);
+	UNUSED_PARAMETER(conn);
+	if (status_code) {
+		*status_code = -1;
+	}
+	return false;
+}
+
 bool
 CivetHandler::handlePut(CivetServer *server, struct mg_connection *conn)
 {
@@ -51,6 +90,19 @@ CivetHandler::handlePut(CivetServer *server, struct mg_connection *conn)
 	return false;
 }
 
+bool
+CivetHandler::handlePut(CivetServer *server,
+                        struct mg_connection *conn,
+                        int *status_code)
+{
+	UNUSED_PARAMETER(server);
+	UNUSED_PARAMETER(conn);
+	if (status_code) {
+		*status_code = -1;
+	}
+	return false;
+}
+
 bool
 CivetHandler::handlePatch(CivetServer *server, struct mg_connection *conn)
 {
@@ -59,6 +111,19 @@ CivetHandler::handlePatch(CivetServer *server, struct mg_connection *conn)
 	return false;
 }
 
+bool
+CivetHandler::handlePatch(CivetServer *server,
+                          struct mg_connection *conn,
+                          int *status_code)
+{
+	UNUSED_PARAMETER(server);
+	UNUSED_PARAMETER(conn);
+	if (status_code) {
+		*status_code = -1;
+	}
+	return false;
+}
+
 bool
 CivetHandler::handleDelete(CivetServer *server, struct mg_connection *conn)
 {
@@ -67,6 +132,19 @@ CivetHandler::handleDelete(CivetServer *server, struct mg_connection *conn)
 	return false;
 }
 
+bool
+CivetHandler::handleDelete(CivetServer *server,
+                           struct mg_connection *conn,
+                           int *status_code)
+{
+	UNUSED_PARAMETER(server);
+	UNUSED_PARAMETER(conn);
+	if (status_code) {
+		*status_code = -1;
+	}
+	return false;
+}
+
 bool
 CivetHandler::handleOptions(CivetServer *server, struct mg_connection *conn)
 {
@@ -75,6 +153,19 @@ CivetHandler::handleOptions(CivetServer *server, struct mg_connection *conn)
 	return false;
 }
 
+bool
+CivetHandler::handleOptions(CivetServer *server,
+                            struct mg_connection *conn,
+                            int *status_code)
+{
+	UNUSED_PARAMETER(server);
+	UNUSED_PARAMETER(conn);
+	if (status_code) {
+		*status_code = -1;
+	}
+	return false;
+}
+
 bool
 CivetWebSocketHandler::handleConnection(CivetServer *server,
                                         const struct mg_connection *conn)
@@ -124,6 +215,8 @@ CivetServer::requestHandler(struct mg_connection *conn, void *cbdata)
 	assert(request_info != NULL);
 	CivetServer *me = (CivetServer *)(request_info->user_data);
 	assert(me != NULL);
+	int http_status_code = -1;
+	bool status_ok = false;
 
 	// Happens when a request hits the server before the context is saved
 	if (me->context == NULL)
@@ -137,23 +230,48 @@ CivetServer::requestHandler(struct mg_connection *conn, void *cbdata)
 
 	if (handler) {
 		if (strcmp(request_info->request_method, "GET") == 0) {
-			return handler->handleGet(me, conn) ? 1 : 0;
+			status_ok = handler->handleGet(me, conn, &http_status_code);
+			if (http_status_code < 0) {
+				status_ok = handler->handleGet(me, conn);
+			}
 		} else if (strcmp(request_info->request_method, "POST") == 0) {
-			return handler->handlePost(me, conn) ? 1 : 0;
+			status_ok = handler->handlePost(me, conn, &http_status_code);
+			if (http_status_code < 0) {
+				status_ok = handler->handlePost(me, conn);
+			}
 		} else if (strcmp(request_info->request_method, "HEAD") == 0) {
-			return handler->handleHead(me, conn) ? 1 : 0;
+			status_ok = handler->handleHead(me, conn, &http_status_code);
+			if (http_status_code < 0) {
+				status_ok = handler->handleHead(me, conn);
+			}
 		} else if (strcmp(request_info->request_method, "PUT") == 0) {
-			return handler->handlePut(me, conn) ? 1 : 0;
+			status_ok = handler->handlePut(me, conn, &http_status_code);
+			if (http_status_code < 0) {
+				status_ok = handler->handlePut(me, conn);
+			}
 		} else if (strcmp(request_info->request_method, "DELETE") == 0) {
-			return handler->handleDelete(me, conn) ? 1 : 0;
+			status_ok = handler->handleDelete(me, conn, &http_status_code);
+			if (http_status_code < 0) {
+				status_ok = handler->handleDelete(me, conn);
+			}
 		} else if (strcmp(request_info->request_method, "OPTIONS") == 0) {
-			return handler->handleOptions(me, conn) ? 1 : 0;
+			status_ok = handler->handleOptions(me, conn, &http_status_code);
+			if (http_status_code < 0) {
+				status_ok = handler->handleOptions(me, conn);
+			}
 		} else if (strcmp(request_info->request_method, "PATCH") == 0) {
-			return handler->handlePatch(me, conn) ? 1 : 0;
+			status_ok = handler->handlePatch(me, conn, &http_status_code);
+			if (http_status_code < 0) {
+				status_ok = handler->handlePatch(me, conn);
+			}
 		}
 	}
 
-	return 0; // No handler found
+	if (http_status_code < 0) {
+		http_status_code = status_ok ? 1 : 0;
+	}
+
+	return http_status_code;
 }
 
 int
@@ -278,6 +396,7 @@ CivetServer::CivetServer(const char **options,
     : context(0)
 {
 	struct CivetCallbacks callbacks;
+	memset(&callbacks, 0, sizeof(callbacks));
 
 	UserContext = UserContextIn;
 
@@ -289,9 +408,10 @@ CivetServer::CivetServer(const char **options,
 	}
 	callbacks.connection_close = closeHandler;
 	context = mg_start(&callbacks, this, options);
-	if (context == NULL)
+	if (context == NULL) {
 		throw CivetException("null context when constructing CivetServer. "
 		                     "Possible problem binding to port.");
+	}
 }
 
 CivetServer::CivetServer(const std::vector<std::string> &options,
@@ -300,6 +420,7 @@ CivetServer::CivetServer(const std::vector<std::string> &options,
     : context(0)
 {
 	struct CivetCallbacks callbacks;
+	memset(&callbacks, 0, sizeof(callbacks));
 
 	UserContext = UserContextIn;
 
@@ -424,6 +545,14 @@ CivetServer::getHeader(struct mg_connection *conn,
 	return mg_get_header(conn, headerName.c_str());
 }
 
+const char *
+CivetServer::getMethod(struct mg_connection *conn)
+{
+	const struct mg_request_info *request_info = mg_get_request_info(conn);
+	assert(request_info != NULL);
+	return request_info->request_method;
+}
+
 void
 CivetServer::urlDecode(const char *src,
                        std::string &dst,
